@@ -80,43 +80,79 @@ import argdb, log
 argdb = argdb.ArgDB(sys.argv)
 log   = log.Log()
 
-# Load classes for packages and process command-line options
-import slepc, petsc, arpack, blzpack, trlan, primme, blopex, sowing, lapack, slicot, hpddm
-slepc   = slepc.SLEPc(argdb,log)
-petsc   = petsc.PETSc(argdb,log)
-arpack  = arpack.Arpack(argdb,log)
-blopex  = blopex.Blopex(argdb,log)
-blzpack = blzpack.Blzpack(argdb,log)
-primme  = primme.Primme(argdb,log)
-trlan   = trlan.Trlan(argdb,log)
-sowing  = sowing.Sowing(argdb,log)
-lapack  = lapack.Lapack(argdb,log)
-slicot  = slicot.Slicot(argdb,log)
-hpddm   = hpddm.HPDDM(argdb,log)
+showhelp = argdb.PopHelp()
 
-externalpackages = [arpack, blopex, blzpack, hpddm, primme, slicot, trlan]
-optionspackages  = [slepc, sowing] + externalpackages
-checkpackages    = externalpackages + [lapack]
+# Load main classes, process the corresponding command-line options
+import slepc, petsc
+slepc = slepc.SLEPc(argdb,log)
+petsc = petsc.PETSc(argdb,log)
+
+# Check enviroment and PETSc version
+if not showhelp:
+  log.Print('Checking environment...')
+  petsc.InitDir(slepc.prefixdir)
+  petsc.LoadVersion()
+slepc.InitDir()
+slepc.LoadVersion()
+
+# Load PETSc configuration
+if not showhelp:
+  petsc.LoadConf()
+  packagesinpetsc = petsc.packages
+else:
+  packagesinpetsc = ''
+
+# Load classes for packages and process their command-line options
+import arpack, blopex, blzpack, elemental, elpa, feast, hpddm, primme, scalapack, slepc4py, slicot, trlan, sowing, lapack
+arpack    = arpack.Arpack(argdb,log)
+blopex    = blopex.Blopex(argdb,log)
+blzpack   = blzpack.Blzpack(argdb,log)
+elemental = elemental.Elemental(argdb,log,packagesinpetsc)
+elpa      = elpa.Elpa(argdb,log)
+feast     = feast.Feast(argdb,log,packagesinpetsc)
+primme    = primme.Primme(argdb,log)
+trlan     = trlan.Trlan(argdb,log)
+sowing    = sowing.Sowing(argdb,log)
+lapack    = lapack.Lapack(argdb,log)
+scalapack = scalapack.Scalapack(argdb,log,packagesinpetsc)
+slepc4py  = slepc4py.Slepc4py(argdb,log)
+slicot    = slicot.Slicot(argdb,log)
+hpddm     = hpddm.HPDDM(argdb,log)
+
+externalpackages = [arpack, blopex, blzpack, elpa, hpddm, primme, slicot, slepc4py, trlan]
+petscpackages    = [lapack, elemental, feast, scalapack]
+checkpackages    = petscpackages + externalpackages
 
 # Print help if requested and check for wrong command-line options
-if argdb.PopHelp():
-  print('SLEPc Configure Help')
-  print('-'*80)
-  for pkg in optionspackages:
+if showhelp:
+  print('\nConfiguration script for SLEPc '+slepc.version)
+  print('\nUsage: ./configure [OPTION]...\n')
+  print('  Brackets indicate an optional part')
+  print('  <bool> means a boolean, use either 0 or 1')
+  print('  <dir> means a directory')
+  print('  <fname> means a file name, can also include the full path or url')
+  print('  <libraries> means a comma-separated list of libraries, e.g., --with-arpack-lib=-lparpack,-larpack')
+  print('  <flags> means a string of flags, e.g., --download-primme-cflags="-std=c99 -g"')
+  slepc.ShowHelp()
+  sowing.ShowHelp()
+  print('\nOptional packages via PETSc (these are tested by default if present in PETSc\'s configuration):\n')
+  for pkg in petscpackages:
     pkg.ShowHelp()
+  print('\nOptional packages (external):\n')
+  for pkg in externalpackages:
+    pkg.ShowHelp()
+  print('')
   sys.exit(0)
 argdb.ErrorPetscOptions()
 argdb.ErrorIfNotEmpty()
 
-# Check enviroment and PETSc version
-log.Print('Checking environment...')
-petsc.InitDir(slepc.prefixdir)
-slepc.InitDir()
-petsc.LoadVersion()
-slepc.LoadVersion()
-
-# Load PETSc configuration
-petsc.LoadConf()
+# Check if packages-download directory contains requested packages
+if slepc.downloaddir:
+  l = list(filter(None, [pkg.MissingTarball(slepc.downloaddir) for pkg in externalpackages]))
+  if l:
+    log.Println('\n\nDownload the following packages and run the script again:')
+    for pkg in l: log.Println(pkg)
+    log.Exit('Missing files in packages-download directory')
 
 # Check for empty PETSC_ARCH
 emptyarch = not ('PETSC_ARCH' in os.environ and os.environ['PETSC_ARCH'])
@@ -188,8 +224,9 @@ includedir = slepc.CreateDir(archdir,'include')
 modulesdir = slepc.CreateDirTwo(confdir,'modules','slepc')
 pkgconfdir = slepc.CreateDir(libdir,'pkgconfig')
 slepcvars  = slepc.CreateFile(confdir,'slepcvariables')
+slepcrules = slepc.CreateFile(confdir,'slepcrules')
 slepcconf  = slepc.CreateFile(includedir,'slepcconf.h')
-pkgconfig  = slepc.CreateFile(pkgconfdir,'SLEPc.pc')
+pkgconfig  = slepc.CreateFile(pkgconfdir,'slepc.pc')
 if slepc.isinstall:
   modules  = slepc.CreateFile(modulesdir,slepc.lversion)
 else:
@@ -231,7 +268,8 @@ if petsc.nversion > slepc.nversion:
 if slepc.release=='1' and not petsc.release=='1':
   log.Exit('A release version of SLEPc requires a release version of PETSc, not a development version')
 if slepc.release=='0' and petsc.release=='1':
-  log.Exit('A development version of SLEPc cannot be built with a release version of PETSc')
+  if not 'slepc' in petsc.packages:
+    log.Exit('A development version of SLEPc cannot be built with a release version of PETSc')
 if petsc.isinstall:
   if os.path.realpath(petsc.prefixdir) != os.path.realpath(petsc.dir):
     log.Warn('PETSC_DIR does not point to PETSc installation path')
@@ -249,7 +287,7 @@ if petsc.singlelib:
 
 # Check for external packages and for missing LAPACK functions
 for pkg in checkpackages:
-  pkg.Process(slepcconf,slepcvars,slepc,petsc,archdir)
+  pkg.Process(slepcconf,slepcvars,slepcrules,slepc,petsc,archdir)
 
 # Write Modules and pkg-config configuration files
 log.NewSection('Writing various configuration files...')
@@ -270,6 +308,7 @@ if not slepc.isinstall:
     log.Exit('Unable to make reconfigure script executable:\n'+str(e))
 
 # Finish with configuration files (except slepcvars)
+slepcrules.close()
 slepcconf.write('\n#endif\n')
 slepcconf.close()
 pkgconfig.close()
@@ -316,7 +355,7 @@ if slepc.isinstall:
 log.Println('PETSc directory:\n '+petsc.dir)
 if petsc.isrepo:
   log.Println('  It is a git repository on branch: '+petsc.branch)
-  if slepc.isrepo and petsc.branch!='maint' and slepc.branch!='maint':
+  if slepc.isrepo and petsc.branch!='release' and slepc.branch!='release':
     try:
       import dateutil.parser, datetime
       petscdate = dateutil.parser.parse(petsc.gitdate)
