@@ -20,7 +20,7 @@ class HPDDM(package.Package):
     package.Package.__init__(self,argdb,log)
     self.packagename    = 'hpddm'
     self.downloadable   = True
-    self.gitcommit      = '21c972f2488edbbd4948c937d7c929b38de9def7'
+    self.gitcommit      = '90bcc19f07d79d3ba032753e22bef25b04cd01b6'
     self.url            = 'https://github.com/hpddm/hpddm/archive/'+self.gitcommit+'.tar.gz'
     self.archive        = self.gitcommit+'.tar.gz'
     self.dirname        = 'hpddm-'+self.gitcommit
@@ -28,41 +28,39 @@ class HPDDM(package.Package):
     self.supports64bint = True
     self.ProcessArgs(argdb)
 
-  def Precondition(self,petsc):
+  def Precondition(self,slepc,petsc):
     pkg = self.packagename.upper()
     if not petsc.cxxdialectcxx11:
       self.log.Exit(pkg+' requires C++11')
     if not petsc.buildsharedlib:
       self.log.Exit(pkg+' requires a shared library build')
-    if petsc.hpddm:
+    if self.packagename in petsc.packages:
       self.log.Exit(pkg+' requires PETSc to be built without '+pkg)
-    package.Package.Precondition(self,petsc)
+    package.Package.Precondition(self,slepc,petsc)
 
-  def DownloadAndInstall(self,conf,vars,slepc,petsc,archdir,prefixdir):
+  def DownloadAndInstall(self,slepcconf,slepcvars,slepc,petsc,archdir,prefixdir):
     externdir = slepc.CreateDir(archdir,'externalpackages')
     builddir  = self.Download(externdir,slepc.downloaddir)
     incdir,libdir = slepc.CreatePrefixDirs(prefixdir)
-    g = open(os.path.join(builddir,'SONAME_SL_LINKER'),'w')
-    g.write('include '+os.path.join(petsc.dir,petsc.arch,'lib','petsc','conf','petscvariables')+'\n')
-    g.write('soname:\n')
-    g.write('\t@echo $(call SONAME_FUNCTION,'+os.path.join(libdir,'libhpddm_petsc')+',0)\n')
-    g.write('sl_linker:\n')
-    g.write('\t@echo $(call SL_LINKER_FUNCTION,'+os.path.join(libdir,'libhpddm_petsc')+',0,0)\n')
-    g.close()
+    cont  = 'include '+os.path.join(petsc.dir,petsc.arch,'lib','petsc','conf','petscvariables')+'\n'
+    cont += 'soname:\n'
+    cont += '\t@echo $(call SONAME_FUNCTION,'+os.path.join(libdir,'libhpddm_petsc')+',0)\n'
+    cont += 'sl_linker:\n'
+    cont += '\t@echo $(call SL_LINKER_FUNCTION,'+os.path.join(libdir,'libhpddm_petsc')+',0,0)\n'
+    self.log.write('Using makefile:\n')
+    self.log.write(cont)
+    mfile = open(os.path.join(builddir,'SONAME_SL_LINKER'),'w')
+    mfile.write(cont)
+    mfile.close()
     d = os.path.join(petsc.dir,petsc.arch,'lib')
     l = petsc.slflag+d+' -L'+d+' -lpetsc'
     d = libdir
     if petsc.isinstall:
-      urlretrieve('https://www.mcs.anl.gov/petsc/petsc-current/src/ksp/ksp/impls/hpddm/hpddm.cxx',os.path.join(builddir,'interface','ksphpddm.cxx'));
-      urlretrieve('https://www.mcs.anl.gov/petsc/petsc-current/src/ksp/pc/impls/hpddm/hpddm.cxx',os.path.join(builddir,'interface','pchpddm.cxx'));
-      urlretrieve('https://www.mcs.anl.gov/petsc/petsc-master/src/ksp/pc/impls/bjacobi/bjacobi.h',os.path.join(builddir,'include','bjacobi.h'));
-      urlretrieve('https://www.mcs.anl.gov/petsc/petsc-master/src/ksp/pc/impls/asm/asm.h',os.path.join(builddir,'include','asm.h'));
-      (result,output) = self.RunCommand('sed -i.bak -e \'s@../src/ksp/pc/impls/bjacobi/@@\' -e \'s@../src/ksp/pc/impls/asm/@@\' '+os.path.join(builddir,'interface','ksphpddm.cxx'))
-      if result:
-        self.log.Exit('Patching petsc/src/ksp/ksp/impls/hpddm/hpddm.cxx failed')
-      result,output = self.RunCommand('sed -i.bak \'s@../src/ksp/pc/impls/asm/@@\' '+os.path.join(builddir,'include','HPDDM_PETSc.hpp'))
-      if result:
-        self.log.Exit('Patching '+os.path.join(builddir,'include','HPDDM_PETSc.hpp')+' failed')
+      branch = 'current'
+      if slepc.isrepo and slepc.branch != 'release':
+        branch = 'master'
+      urlretrieve('https://www.mcs.anl.gov/petsc/petsc-'+branch+'/src/ksp/ksp/impls/hpddm/hpddm.cxx',os.path.join(builddir,'interface','ksphpddm.cxx'));
+      urlretrieve('https://www.mcs.anl.gov/petsc/petsc-'+branch+'/src/ksp/pc/impls/hpddm/hpddm.cxx',os.path.join(builddir,'interface','pchpddm.cxx'));
     else:
       shutil.copyfile(os.path.join(petsc.dir,'src','ksp','ksp','impls','hpddm','hpddm.cxx'),os.path.join(builddir,'interface','ksphpddm.cxx'))
       shutil.copyfile(os.path.join(petsc.dir,'src','ksp','pc','impls','hpddm','hpddm.cxx'),os.path.join(builddir,'interface','pchpddm.cxx'))
@@ -83,9 +81,9 @@ class HPDDM(package.Package):
     l = petsc.slflag+d+' -L'+d+' -lhpddm_petsc'
     f = '-I'+incdir
     # Write configuration files
-    conf.write('#define SLEPC_HAVE_HPDDM 1\n')
-    vars.write('HPDDM_LIB = '+l+'\n')
-    vars.write('HPDDM_INCLUDE = '+f+'\n')
+    slepcconf.write('#define SLEPC_HAVE_HPDDM 1\n')
+    slepcvars.write('HPDDM_LIB = '+l+'\n')
+    slepcvars.write('HPDDM_INCLUDE = '+f+'\n')
     self.packageflags = [l] + [f]
     self.havepackage = True
 
