@@ -25,7 +25,7 @@ class Slicot(package.Package):
     self.ProcessArgs(argdb)
 
 
-  def Check(self,conf,vars,petsc,archdir):
+  def Check(self,slepcconf,slepcvars,petsc,archdir):
     functions = ['sb03od','sb03md']
     if self.packagelibs:
       libs = [self.packagelibs]
@@ -37,22 +37,25 @@ class Slicot(package.Package):
     else:
       dirs = self.GenerateGuesses('slicot',archdir)
 
-    self.FortranLib(conf,vars,dirs,libs,functions)
+    self.FortranLib(slepcconf,slepcvars,dirs,libs,functions)
 
 
-  def DownloadAndInstall(self,conf,vars,slepc,petsc,archdir,prefixdir):
+  def DownloadAndInstall(self,slepcconf,slepcvars,slepc,petsc,archdir,prefixdir):
     externdir = slepc.CreateDir(archdir,'externalpackages')
     builddir  = self.Download(externdir,slepc.downloaddir)
     libname = 'libslicot.a'
 
-    # Configure
-    g = open(os.path.join(builddir,'make.inc'),'w')
-    g.write('FORTRAN   = '+petsc.fc+'\n')
-    g.write('OPTS      = '+petsc.fc_flags.replace('-Wall','').replace('-Wshadow','')+'\n')
-    g.write('ARCH      = '+petsc.ar+'\n')
-    g.write('ARCHFLAGS = '+petsc.ar_flags+'\n')
-    g.write('SLICOTLIB = ../'+libname+'\n')
-    g.close()
+    # Makefile
+    cont  = 'FORTRAN   = '+petsc.fc+'\n'
+    cont += 'OPTS      = '+petsc.fc_flags.replace('-Wall','').replace('-Wshadow','')+'\n'
+    cont += 'ARCH      = '+petsc.ar+'\n'
+    cont += 'ARCHFLAGS = '+petsc.ar_flags+'\n'
+    cont += 'SLICOTLIB = ../'+libname+'\n'
+    self.log.write('Using makefile definitions:\n')
+    self.log.write(cont)
+    mfile = open(os.path.join(builddir,'make.inc'),'w')
+    mfile.write(cont)
+    mfile.close()
 
     # Build package
     target = 'lib'
@@ -68,5 +71,5 @@ class Slicot(package.Package):
     functions = ['sb03od']
     libs = [['-lslicot']]
     dirs = [libdir]
-    self.FortranLib(conf,vars,dirs,libs,functions)
+    self.FortranLib(slepcconf,slepcvars,dirs,libs,functions)
 
