@@ -73,6 +73,31 @@ PetscErrorCode EPSSetDefaultST_GMRES(EPS eps)
   PetscFunctionReturn(0);
 }
 
+/*
+   This is for direct eigensolvers that work with A and B directly, so
+   no need to factorize B.
+*/
+PetscErrorCode EPSSetDefaultST_NoFactor(EPS eps)
+{
+  PetscErrorCode ierr;
+  KSP            ksp;
+  PC             pc;
+
+  PetscFunctionBegin;
+  if (!((PetscObject)eps->st)->type_name) {
+    ierr = STSetType(eps->st,STSHIFT);CHKERRQ(ierr);
+  }
+  ierr = STGetKSP(eps->st,&ksp);CHKERRQ(ierr);
+  if (!((PetscObject)ksp)->type_name) {
+    ierr = KSPSetType(ksp,KSPPREONLY);CHKERRQ(ierr);
+  }
+  ierr = KSPGetPC(ksp,&pc);CHKERRQ(ierr);
+  if (!((PetscObject)pc)->type_name) {
+    ierr = PCSetType(pc,PCNONE);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
 /*
    Check that the ST selected by the user is compatible with the EPS solver and options
 */
@@ -114,114 +139,12 @@ PetscErrorCode EPSCheckCompatibleST(EPS eps)
   PetscFunctionReturn(0);
 }
 
-/*@
-   EPSSetUp - Sets up all the internal data structures necessary for the
-   execution of the eigensolver. Then calls STSetUp() for any set-up
-   operations associated to the ST object.
-
-   Collective on eps
-
-   Input Parameter:
-.  eps   - eigenproblem solver context
-
-   Notes:
-   This function need not be called explicitly in most cases, since EPSSolve()
-   calls it. It can be useful when one wants to measure the set-up time
-   separately from the solve time.
-
-   Level: developer
-
-.seealso: EPSCreate(), EPSSolve(), EPSDestroy(), STSetUp(), EPSSetInitialSpace()
-@*/
-PetscErrorCode EPSSetUp(EPS eps)
+/*
+   EPSSetUpSort_Basic: configure the EPS sorting criterion according to 'which'
+*/
+PetscErrorCode EPSSetUpSort_Basic(EPS eps)
 {
-  PetscErrorCode ierr;
-  Mat            A,B;
-  SlepcSC        sc;
-  PetscInt       k,nmat;
-  PetscBool      flg,istrivial;
-
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  if (eps->state) PetscFunctionReturn(0);
-  ierr = PetscLogEventBegin(EPS_SetUp,eps,0,0,0);CHKERRQ(ierr);
-
-  /* reset the convergence flag from the previous solves */
-  eps->reason = EPS_CONVERGED_ITERATING;
-
-  /* Set default solver type (EPSSetFromOptions was not called) */
-  if (!((PetscObject)eps)->type_name) {
-    ierr = EPSSetType(eps,EPSKRYLOVSCHUR);CHKERRQ(ierr);
-  }
-  if (!eps->st) { ierr = EPSGetST(eps,&eps->st);CHKERRQ(ierr); }
-  ierr = EPSSetDefaultST(eps);CHKERRQ(ierr);
-
-  ierr = STSetTransform(eps->st,PETSC_TRUE);CHKERRQ(ierr);
-  if (eps->useds && !eps->ds) { ierr = EPSGetDS(eps,&eps->ds);CHKERRQ(ierr); }
-  if (eps->twosided) {
-    if (!eps->hasts) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver does not support computing left eigenvectors (no two-sided variant)");
-    if (eps->ishermitian) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Two-sided methods are not intended for symmetric problems");
-    if (!eps->dsts) { ierr = DSDuplicate(eps->ds,&eps->dsts);CHKERRQ(ierr); }
-  }
-  if (!eps->rg) { ierr = EPSGetRG(eps,&eps->rg);CHKERRQ(ierr); }
-  if (!((PetscObject)eps->rg)->type_name) {
-    ierr = RGSetType(eps->rg,RGINTERVAL);CHKERRQ(ierr);
-  }
-
-  /* Set problem dimensions */
-  ierr = STGetNumMatrices(eps->st,&nmat);CHKERRQ(ierr);
-  if (!nmat) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONGSTATE,"EPSSetOperators must be called first");
-  ierr = STMatGetSize(eps->st,&eps->n,NULL);CHKERRQ(ierr);
-  ierr = STMatGetLocalSize(eps->st,&eps->nloc,NULL);CHKERRQ(ierr);
-
-  /* Set default problem type */
-  if (!eps->problem_type) {
-    if (nmat==1) {
-      ierr = EPSSetProblemType(eps,EPS_NHEP);CHKERRQ(ierr);
-    } else {
-      ierr = EPSSetProblemType(eps,EPS_GNHEP);CHKERRQ(ierr);
-    }
-  } else if (nmat==1 && eps->isgeneralized) {
-    ierr = PetscInfo(eps,"Eigenproblem set as generalized but no matrix B was provided; reverting to a standard eigenproblem\n");CHKERRQ(ierr);
-    eps->isgeneralized = PETSC_FALSE;
-    eps->problem_type = eps->ishermitian? EPS_HEP: EPS_NHEP;
-  } else if (nmat>1 && !eps->isgeneralized) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_INCOMP,"Inconsistent EPS state");
-
-  if (eps->nev > eps->n) eps->nev = eps->n;
-  if (eps->ncv > eps->n) eps->ncv = eps->n;
-
-  /* initialization of matrix norms */
-  if (eps->conv==EPS_CONV_NORM) {
-    if (!eps->nrma) {
-      ierr = STGetMatrix(eps->st,0,&A);CHKERRQ(ierr);
-      ierr = MatNorm(A,NORM_INFINITY,&eps->nrma);CHKERRQ(ierr);
-    }
-    if (nmat>1 && !eps->nrmb) {
-      ierr = STGetMatrix(eps->st,1,&B);CHKERRQ(ierr);
-      ierr = MatNorm(B,NORM_INFINITY,&eps->nrmb);CHKERRQ(ierr);
-    }
-  }
-
-  /* call specific solver setup */
-  ierr = (*eps->ops->setup)(eps);CHKERRQ(ierr);
-
-  /* if purification is set, check that it really makes sense */
-  if (eps->purify) {
-    if (eps->categ==EPS_CATEGORY_PRECOND || eps->categ==EPS_CATEGORY_CONTOUR) eps->purify = PETSC_FALSE;
-    else {
-      if (!eps->isgeneralized) eps->purify = PETSC_FALSE;
-      else if (!eps->ishermitian && !eps->ispositive) eps->purify = PETSC_FALSE;
-      else {
-        ierr = PetscObjectTypeCompare((PetscObject)eps->st,STCAYLEY,&flg);CHKERRQ(ierr);
-        if (flg) eps->purify = PETSC_FALSE;
-      }
-    }
-  }
-
-  /* set tolerance if not yet set */
-  if (eps->tol==PETSC_DEFAULT) eps->tol = SLEPC_DEFAULT_TOL;
-
-  /* fill sorting criterion context */
   switch (eps->which) {
     case EPS_LARGEST_MAGNITUDE:
       eps->sc->comparison    = SlepcCompareLargestMagnitude;
@@ -270,19 +193,146 @@ PetscErrorCode EPSSetUp(EPS eps)
   }
   eps->sc->map    = NULL;
   eps->sc->mapobj = NULL;
+  PetscFunctionReturn(0);
+}
 
+/*
+   EPSSetUpSort_Default: configure both EPS and DS sorting criterion
+*/
+PetscErrorCode EPSSetUpSort_Default(EPS eps)
+{
+  PetscErrorCode ierr;
+  SlepcSC        sc;
+  PetscBool      istrivial;
+
+  PetscFunctionBegin;
+  /* fill sorting criterion context */
+  ierr = EPSSetUpSort_Basic(eps);CHKERRQ(ierr);
   /* fill sorting criterion for DS */
-  if (eps->useds && eps->which!=EPS_ALL) {
-    ierr = DSGetSlepcSC(eps->ds,&sc);CHKERRQ(ierr);
-    ierr = RGIsTrivial(eps->rg,&istrivial);CHKERRQ(ierr);
-    sc->rg            = istrivial? NULL: eps->rg;
-    sc->comparison    = eps->sc->comparison;
-    sc->comparisonctx = eps->sc->comparisonctx;
-    sc->map           = SlepcMap_ST;
-    sc->mapobj        = (PetscObject)eps->st;
-    if (eps->twosided) {
-      ierr = DSSetSlepcSC(eps->dsts,sc);CHKERRQ(ierr);
+  ierr = DSGetSlepcSC(eps->ds,&sc);CHKERRQ(ierr);
+  ierr = RGIsTrivial(eps->rg,&istrivial);CHKERRQ(ierr);
+  sc->rg            = istrivial? NULL: eps->rg;
+  sc->comparison    = eps->sc->comparison;
+  sc->comparisonctx = eps->sc->comparisonctx;
+  sc->map           = SlepcMap_ST;
+  sc->mapobj        = (PetscObject)eps->st;
+  if (eps->twosided) {
+    ierr = DSSetSlepcSC(eps->dsts,sc);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
+/*@
+   EPSSetUp - Sets up all the internal data structures necessary for the
+   execution of the eigensolver. Then calls STSetUp() for any set-up
+   operations associated to the ST object.
+
+   Collective on eps
+
+   Input Parameter:
+.  eps   - eigenproblem solver context
+
+   Notes:
+   This function need not be called explicitly in most cases, since EPSSolve()
+   calls it. It can be useful when one wants to measure the set-up time
+   separately from the solve time.
+
+   Level: developer
+
+.seealso: EPSCreate(), EPSSolve(), EPSDestroy(), STSetUp(), EPSSetInitialSpace()
+@*/
+PetscErrorCode EPSSetUp(EPS eps)
+{
+  PetscErrorCode ierr;
+  Mat            A,B;
+  PetscInt       k,nmat;
+  PetscBool      flg;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  if (eps->state) PetscFunctionReturn(0);
+  ierr = PetscLogEventBegin(EPS_SetUp,eps,0,0,0);CHKERRQ(ierr);
+
+  /* reset the convergence flag from the previous solves */
+  eps->reason = EPS_CONVERGED_ITERATING;
+
+  /* Set default solver type (EPSSetFromOptions was not called) */
+  if (!((PetscObject)eps)->type_name) {
+    ierr = EPSSetType(eps,EPSKRYLOVSCHUR);CHKERRQ(ierr);
+  }
+  if (!eps->st) { ierr = EPSGetST(eps,&eps->st);CHKERRQ(ierr); }
+  ierr = EPSSetDefaultST(eps);CHKERRQ(ierr);
+
+  ierr = STSetTransform(eps->st,PETSC_TRUE);CHKERRQ(ierr);
+  if (eps->useds && !eps->ds) { ierr = EPSGetDS(eps,&eps->ds);CHKERRQ(ierr); }
+  if (eps->twosided) {
+    if (eps->ishermitian && (!eps->isgeneralized || eps->ispositive)) SETERRQ1(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Two-sided methods are not intended for %s problems",SLEPC_STRING_HERMITIAN);
+    if (!eps->dsts) { ierr = DSDuplicate(eps->ds,&eps->dsts);CHKERRQ(ierr); }
+  }
+  if (!eps->rg) { ierr = EPSGetRG(eps,&eps->rg);CHKERRQ(ierr); }
+  if (!((PetscObject)eps->rg)->type_name) {
+    ierr = RGSetType(eps->rg,RGINTERVAL);CHKERRQ(ierr);
+  }
+
+  /* Set problem dimensions */
+  ierr = STGetNumMatrices(eps->st,&nmat);CHKERRQ(ierr);
+  if (!nmat) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONGSTATE,"EPSSetOperators must be called first");
+  ierr = STMatGetSize(eps->st,&eps->n,NULL);CHKERRQ(ierr);
+  ierr = STMatGetLocalSize(eps->st,&eps->nloc,NULL);CHKERRQ(ierr);
+
+  /* Set default problem type */
+  if (!eps->problem_type) {
+    if (nmat==1) {
+      ierr = EPSSetProblemType(eps,EPS_NHEP);CHKERRQ(ierr);
+    } else {
+      ierr = EPSSetProblemType(eps,EPS_GNHEP);CHKERRQ(ierr);
     }
+  } else if (nmat==1 && eps->isgeneralized) {
+    ierr = PetscInfo(eps,"Eigenproblem set as generalized but no matrix B was provided; reverting to a standard eigenproblem\n");CHKERRQ(ierr);
+    eps->isgeneralized = PETSC_FALSE;
+    eps->problem_type = eps->ishermitian? EPS_HEP: EPS_NHEP;
+  } else if (nmat>1 && !eps->isgeneralized) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_INCOMP,"Inconsistent EPS state: the problem type does not match the number of matrices");
+
+  if (eps->nev > eps->n) eps->nev = eps->n;
+  if (eps->ncv > eps->n) eps->ncv = eps->n;
+
+  /* check some combinations of eps->which */
+  if (eps->ishermitian && (!eps->isgeneralized || eps->ispositive) && (eps->which==EPS_LARGEST_IMAGINARY || eps->which==EPS_SMALLEST_IMAGINARY || eps->which==EPS_TARGET_IMAGINARY)) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Sorting the eigenvalues along the imaginary axis is not allowed when all eigenvalues are real");
+
+  /* initialization of matrix norms */
+  if (eps->conv==EPS_CONV_NORM) {
+    if (!eps->nrma) {
+      ierr = STGetMatrix(eps->st,0,&A);CHKERRQ(ierr);
+      ierr = MatNorm(A,NORM_INFINITY,&eps->nrma);CHKERRQ(ierr);
+    }
+    if (nmat>1 && !eps->nrmb) {
+      ierr = STGetMatrix(eps->st,1,&B);CHKERRQ(ierr);
+      ierr = MatNorm(B,NORM_INFINITY,&eps->nrmb);CHKERRQ(ierr);
+    }
+  }
+
+  /* call specific solver setup */
+  ierr = (*eps->ops->setup)(eps);CHKERRQ(ierr);
+
+  /* if purification is set, check that it really makes sense */
+  if (eps->purify) {
+    if (eps->categ==EPS_CATEGORY_PRECOND || eps->categ==EPS_CATEGORY_CONTOUR) eps->purify = PETSC_FALSE;
+    else {
+      if (!eps->isgeneralized) eps->purify = PETSC_FALSE;
+      else if (!eps->ishermitian && !eps->ispositive) eps->purify = PETSC_FALSE;
+      else {
+        ierr = PetscObjectTypeCompare((PetscObject)eps->st,STCAYLEY,&flg);CHKERRQ(ierr);
+        if (flg) eps->purify = PETSC_FALSE;
+      }
+    }
+  }
+
+  /* set tolerance if not yet set */
+  if (eps->tol==PETSC_DEFAULT) eps->tol = SLEPC_DEFAULT_TOL;
+
+  /* set up sorting criterion */
+  if (eps->ops->setupsort) {
+    ierr = (*eps->ops->setupsort)(eps);CHKERRQ(ierr);
   }
 
   /* Build balancing matrix if required */
