@@ -121,7 +121,7 @@ int main (int argc,char **argv)
     ierr = EPSErrorView(eps,EPS_ERROR_RELATIVE,NULL);CHKERRQ(ierr);
   } else {
     ierr = PetscViewerPushFormat(PETSC_VIEWER_STDOUT_WORLD,PETSC_VIEWER_ASCII_INFO_DETAIL);CHKERRQ(ierr);
-    ierr = EPSReasonView(eps,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
+    ierr = EPSConvergedReasonView(eps,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
     ierr = EPSErrorView(eps,EPS_ERROR_RELATIVE,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
     ierr = PetscViewerPopFormat(PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
   }
@@ -204,15 +204,8 @@ PetscErrorCode STDestroy_Fold(FoldShellST *fold)
 
 /*TEST
 
-   testset:
+   test:
       args: -m 11 -eps_nev 4 -terse
-      output_file: output/ex35_1.out
-      test:
-         suffix: 1
-         requires: !single
-      test:
-         suffix: 1_single
-         args: -eps_tol 1e-5
-         requires: single
+      suffix: 1
 
 TEST*/
