@@ -887,7 +887,7 @@ PetscErrorCode BVSetFromOptions(BV bv)
   PetscValidHeaderSpecific(bv,BV_CLASSID,1);
   ierr = BVRegisterAll();CHKERRQ(ierr);
   ierr = PetscObjectOptionsBegin((PetscObject)bv);CHKERRQ(ierr);
-    ierr = PetscOptionsFList("-bv_type","Basis Vectors type","BVSetType",BVList,(char*)(((PetscObject)bv)->type_name?((PetscObject)bv)->type_name:BVSVEC),type,256,&flg1);CHKERRQ(ierr);
+    ierr = PetscOptionsFList("-bv_type","Basis Vectors type","BVSetType",BVList,(char*)(((PetscObject)bv)->type_name?((PetscObject)bv)->type_name:BVSVEC),type,sizeof(type),&flg1);CHKERRQ(ierr);
     if (flg1) {
       ierr = BVSetType(bv,type);CHKERRQ(ierr);
     } else if (!((PetscObject)bv)->type_name) {
@@ -1451,25 +1451,29 @@ PetscErrorCode BVGetMat(BV bv,Mat *A)
   PetscValidHeaderSpecific(bv,BV_CLASSID,1);
   BVCheckSizes(bv,1);
   PetscValidPointer(A,2);
-  m = bv->k-bv->l;
-  if (!bv->Aget) create=PETSC_TRUE;
-  else {
-    ierr = MatDenseGetArray(bv->Aget,&aa);CHKERRQ(ierr);
-    if (aa) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_WRONGSTATE,"BVGetMat already called on this BV");
-    ierr = MatGetSize(bv->Aget,NULL,&cols);CHKERRQ(ierr);
-    if (cols!=m) {
-      ierr = MatDestroy(&bv->Aget);CHKERRQ(ierr);
-      create=PETSC_TRUE;
+  if (bv->ops->getmat) {
+    ierr = (*bv->ops->getmat)(bv,A);CHKERRQ(ierr);
+  } else {
+    m = bv->k-bv->l;
+    if (!bv->Aget) create=PETSC_TRUE;
+    else {
+      ierr = MatDenseGetArray(bv->Aget,&aa);CHKERRQ(ierr);
+      if (aa) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_WRONGSTATE,"BVGetMat already called on this BV");
+      ierr = MatGetSize(bv->Aget,NULL,&cols);CHKERRQ(ierr);
+      if (cols!=m) {
+        ierr = MatDestroy(&bv->Aget);CHKERRQ(ierr);
+        create=PETSC_TRUE;
+      }
     }
+    ierr = BVGetArray(bv,&vv);CHKERRQ(ierr);
+    if (create) {
+      ierr = MatCreateDense(PetscObjectComm((PetscObject)bv),bv->n,PETSC_DECIDE,bv->N,m,vv,&bv->Aget);CHKERRQ(ierr); /* pass a pointer to avoid allocation of storage */
+      ierr = MatDenseReplaceArray(bv->Aget,NULL);CHKERRQ(ierr);  /* replace with a null pointer, the value after BVRestoreMat */
+      ierr = PetscLogObjectParent((PetscObject)bv,(PetscObject)bv->Aget);CHKERRQ(ierr);
+    }
+    ierr = MatDensePlaceArray(bv->Aget,vv+(bv->nc+bv->l)*bv->n);CHKERRQ(ierr);  /* set the actual pointer */
+    *A = bv->Aget;
   }
-  ierr = BVGetArray(bv,&vv);CHKERRQ(ierr);
-  if (create) {
-    ierr = MatCreateDense(PetscObjectComm((PetscObject)bv),bv->n,PETSC_DECIDE,bv->N,m,vv,&bv->Aget);CHKERRQ(ierr); /* pass a pointer to avoid allocation of storage */
-    ierr = MatDensePlaceArray(bv->Aget,NULL);CHKERRQ(ierr);  /* replace with a null pointer, the value after BVRestoreMat */
-    ierr = PetscLogObjectParent((PetscObject)bv,(PetscObject)bv->Aget);CHKERRQ(ierr);
-  }
-  ierr = MatDensePlaceArray(bv->Aget,vv+(bv->nc+bv->l)*bv->n);CHKERRQ(ierr);  /* set the actual pointer */
-  *A = bv->Aget;
   PetscFunctionReturn(0);
 }
 
@@ -1502,11 +1506,15 @@ PetscErrorCode BVRestoreMat(BV bv,Mat *A)
   PetscValidPointer(A,2);
   if (!bv->Aget) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_WRONGSTATE,"BVRestoreMat must match a previous call to BVGetMat");
   if (bv->Aget!=*A) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_WRONGSTATE,"Mat argument is not the same as the one obtained with BVGetMat");
-  ierr = MatDenseGetArray(bv->Aget,&aa);CHKERRQ(ierr);
-  vv = aa-(bv->nc+bv->l)*bv->n;
-  ierr = MatDenseResetArray(bv->Aget);CHKERRQ(ierr);
-  ierr = BVRestoreArray(bv,&vv);CHKERRQ(ierr);
-  *A = NULL;
+  if (bv->ops->restoremat) {
+    ierr = (*bv->ops->restoremat)(bv,A);CHKERRQ(ierr);
+  } else {
+    ierr = MatDenseGetArray(bv->Aget,&aa);CHKERRQ(ierr);
+    vv = aa-(bv->nc+bv->l)*bv->n;
+    ierr = MatDenseResetArray(bv->Aget);CHKERRQ(ierr);
+    ierr = BVRestoreArray(bv,&vv);CHKERRQ(ierr);
+    *A = NULL;
+  }
   PetscFunctionReturn(0);
 }
 
@@ -1686,7 +1694,7 @@ PetscErrorCode BVCopy(BV V,BV W)
   PetscCheckSameTypeAndComm(V,1,W,2);
   if (V->n!=W->n) SETERRQ2(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_INCOMP,"Mismatching local dimension V %D, W %D",V->n,W->n);
   if (V->k-V->l>W->m-W->l) SETERRQ2(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_SIZ,"W has %D non-leading columns, not enough to store %D columns",W->m-W->l,V->k-V->l);
-  if (!V->n) PetscFunctionReturn(0);
+  if (V==W || !V->n) PetscFunctionReturn(0);
 
   ierr = PetscLogEventBegin(BV_Copy,V,W,0,0);CHKERRQ(ierr);
   if (V->indef && V->matrix && V->indef==W->indef && V->matrix==W->matrix) {
@@ -1804,6 +1812,7 @@ static PetscErrorCode BVGetSplit_Private(BV bv,PetscBool left,BV *split)
 
   PetscFunctionBegin;
   ncols = left? bv->nc+bv->l: bv->m-bv->l;
+  if (*split && ncols!=(*split)->m) { ierr = BVDestroy(split);CHKERRQ(ierr); }
   if (!*split) {
     ierr = BVCreate(PetscObjectComm((PetscObject)bv),split);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)bv,(PetscObject)*split);CHKERRQ(ierr);
@@ -1811,8 +1820,6 @@ static PetscErrorCode BVGetSplit_Private(BV bv,PetscBool left,BV *split)
     (*split)->splitparent = bv;
     ierr = BVSetSizesFromVec(*split,bv->t,ncols);CHKERRQ(ierr);
     ierr = BVDuplicate_Private(bv,*split);CHKERRQ(ierr);
-  } else {
-    ierr = BVResize(*split,ncols,PETSC_FALSE);CHKERRQ(ierr);
   }
   (*split)->l  = 0;
   (*split)->k  = left? bv->l: bv->k-bv->l;
