@@ -14,6 +14,23 @@
 #include <slepc/private/bvimpl.h>
 #include <slepcblaslapack.h>
 
+/*
+    Reduction operation to compute sqrt(x**2+y**2) when normalizing vectors
+*/
+SLEPC_EXTERN void MPIAPI SlepcPythag(void *in,void *inout,PetscMPIInt *len,MPI_Datatype *datatype)
+{
+  PetscBLASInt i,n=*len;
+  PetscReal    *x = (PetscReal*)in,*y = (PetscReal*)inout;
+
+  PetscFunctionBegin;
+  if (PetscUnlikely(*datatype!=MPIU_REAL)) {
+    (*PetscErrorPrintf)("Only implemented for MPIU_REAL data type");
+    MPI_Abort(MPI_COMM_WORLD,1);
+  }
+  for (i=0;i<n;i++) y[i] = SlepcAbs(x[i],y[i]);
+  PetscFunctionReturnVoid();
+}
+
 /*
     Compute ||A|| for an mxn matrix
 */
@@ -31,9 +48,7 @@ PetscErrorCode BVNorm_LAPACK_Private(BV bv,PetscInt m_,PetscInt n_,const PetscSc
   if (type==NORM_FROBENIUS || type==NORM_2) {
     lnrm = LAPACKlange_("F",&m,&n,(PetscScalar*)A,&m,rwork);
     if (mpi) {
-      lnrm = lnrm*lnrm;
-      ierr = MPI_Allreduce(&lnrm,nrm,1,MPIU_REAL,MPIU_SUM,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
-      *nrm = PetscSqrtReal(*nrm);
+      ierr = MPIU_Allreduce(&lnrm,nrm,1,MPIU_REAL,MPIU_LAPY2,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
     } else *nrm = lnrm;
     ierr = PetscLogFlops(2.0*m*n);CHKERRQ(ierr);
   } else if (type==NORM_1) {
@@ -49,7 +64,7 @@ PetscErrorCode BVNorm_LAPACK_Private(BV bv,PetscInt m_,PetscInt n_,const PetscSc
         }
       }
       ierr = PetscMPIIntCast(n_,&len);CHKERRQ(ierr);
-      ierr = MPI_Allreduce(rwork,rwork2,len,MPIU_REAL,MPIU_SUM,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
+      ierr = MPIU_Allreduce(rwork,rwork2,len,MPIU_REAL,MPIU_SUM,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
       *nrm = 0.0;
       for (j=0;j<n_;j++) if (rwork2[j] > *nrm) *nrm = rwork2[j];
     } else {
@@ -61,7 +76,7 @@ PetscErrorCode BVNorm_LAPACK_Private(BV bv,PetscInt m_,PetscInt n_,const PetscSc
     rwork = (PetscReal*)bv->work;
     lnrm = LAPACKlange_("I",&m,&n,(PetscScalar*)A,&m,rwork);
     if (mpi) {
-      ierr = MPI_Allreduce(&lnrm,nrm,1,MPIU_REAL,MPIU_MAX,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
+      ierr = MPIU_Allreduce(&lnrm,nrm,1,MPIU_REAL,MPIU_MAX,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
     } else *nrm = lnrm;
     ierr = PetscLogFlops(1.0*m*n);CHKERRQ(ierr);
   }
@@ -69,6 +84,53 @@ PetscErrorCode BVNorm_LAPACK_Private(BV bv,PetscInt m_,PetscInt n_,const PetscSc
   PetscFunctionReturn(0);
 }
 
+/*
+    Normalize the columns of an mxn matrix A
+*/
+PetscErrorCode BVNormalize_LAPACK_Private(BV bv,PetscInt m_,PetscInt n_,const PetscScalar *A,PetscScalar *eigi,PetscBool mpi)
+{
+  PetscErrorCode ierr;
+  PetscBLASInt   m,j,k,info,zero=0;
+  PetscMPIInt    len;
+  PetscReal      *norms,*rwork=NULL,*rwork2=NULL,done=1.0;
+
+  PetscFunctionBegin;
+  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
+  ierr = PetscBLASIntCast(m_,&m);CHKERRQ(ierr);
+  ierr = BVAllocateWork_Private(bv,2*n_);CHKERRQ(ierr);
+  rwork = (PetscReal*)bv->work;
+  rwork2 = rwork+n_;
+  /* compute local norms */
+  for (j=0;j<n_;j++) {
+    k = 1;
+#if !defined(PETSC_USE_COMPLEX)
+    if (eigi && eigi[j] != 0.0) k = 2;
+#endif
+    rwork[j] = LAPACKlange_("F",&m,&k,(PetscScalar*)(A+j*m_),&m,rwork2);
+    if (k==2) { rwork[j+1] = rwork[j]; j++; }
+  }
+  /* reduction to get global norms */
+  if (mpi) {
+    ierr = PetscMPIIntCast(n_,&len);CHKERRQ(ierr);
+    ierr = PetscArrayzero(rwork2,n_);CHKERRQ(ierr);
+    ierr = MPIU_Allreduce(rwork,rwork2,len,MPIU_REAL,MPIU_LAPY2,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
+    norms = rwork2;
+  } else norms = rwork;
+  /* scale columns */
+  for (j=0;j<n_;j++) {
+    k = 1;
+#if !defined(PETSC_USE_COMPLEX)
+    if (eigi && eigi[j] != 0.0) k = 2;
+#endif
+    PetscStackCallBLAS("LAPACKlascl",LAPACKlascl_("G",&zero,&zero,norms+j,&done,&m,&k,(PetscScalar*)(A+j*m_),&m,&info));
+    SlepcCheckLapackInfo("lascl",info);
+    if (k==2) j++;
+  }
+  ierr = PetscLogFlops(3.0*m*n_);CHKERRQ(ierr);
+  ierr = PetscFPTrapPop();CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*
    Compute the upper Cholesky factor in R and its inverse in S.
    If S == R then the inverse overwrites the Cholesky factor.
@@ -78,7 +140,7 @@ PetscErrorCode BVMatCholInv_LAPACK_Private(BV bv,Mat R,Mat S)
   PetscErrorCode ierr;
   PetscInt       i,k,l,n,m,ld,lds;
   PetscScalar    *pR,*pS;
-  PetscBLASInt   info,n_,m_,ld_,lds_;
+  PetscBLASInt   info,n_ = 0,m_ = 0,ld_,lds_;
 
   PetscFunctionBegin;
   l = bv->l;
@@ -153,7 +215,7 @@ PetscErrorCode BVMatTriInv_LAPACK_Private(BV bv,Mat R,Mat S)
   PetscErrorCode ierr;
   PetscInt       i,k,l,n,m,ld,lds;
   PetscScalar    *pR,*pS;
-  PetscBLASInt   info,n_,m_,ld_,lds_;
+  PetscBLASInt   info,n_,m_ = 0,ld_,lds_;
 
   PetscFunctionBegin;
   l = bv->l;
@@ -208,7 +270,7 @@ PetscErrorCode BVMatSVQB_LAPACK_Private(BV bv,Mat R,Mat S)
   PetscInt       i,j,k,l,n,m,ld,lds;
   PetscScalar    *pR,*pS,*D,*work,a;
   PetscReal      *eig,dummy;
-  PetscBLASInt   info,lwork,n_,m_,ld_,lds_;
+  PetscBLASInt   info,lwork,n_,m_ = 0,ld_,lds_;
 #if defined(PETSC_USE_COMPLEX)
   PetscReal      *rwork,rdummy;
 #endif
@@ -291,7 +353,7 @@ PetscErrorCode BVOrthogonalize_LAPACK_TSQR(BV bv,PetscInt m_,PetscInt n_,PetscSc
 {
   PetscErrorCode ierr;
   PetscInt       level,plevel,nlevels,powtwo,lda,worklen;
-  PetscBLASInt   m,n,i,j,k,l,s,nb,sz,lwork,info;
+  PetscBLASInt   m,n,i,j,k,l,s = 0,nb,sz,lwork,info;
   PetscScalar    *tau,*work,*A=NULL,*QQ=NULL,*Qhalf,*C=NULL,one=1.0,zero=0.0;
   PetscMPIInt    rank,size,count,stride;
   MPI_Datatype   tmat;
@@ -495,7 +557,7 @@ PetscErrorCode BVOrthogonalize_LAPACK_TSQR_OnlyR(BV bv,PetscInt m_,PetscInt n_,P
     for (i=0;i<n;i++) {
       for (j=i;j<n;j++) R1[(2*n-i-1)*i/2+j] = (i<m)?A[i+j*m]:0.0;
     }
-    ierr = MPI_Allreduce(R1,R2,1,tmat,MPIU_TSQR,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
+    ierr = MPIU_Allreduce(R1,R2,1,tmat,MPIU_TSQR,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
     for (i=0;i<n;i++) {
       for (j=0;j<i;j++) R[i+j*ldr] = 0.0;
       for (j=i;j<n;j++) R[i+j*ldr] = R2[(2*n-i-1)*i/2+j];
